@@ -152,6 +152,36 @@ import base64
 from itertools import chain
 import os, errno
 
+
+def decider_tgt_src_content(dep, target, prev_ni):
+    """
+    A decider that fixes some warts in the SCons decider. It triggers a restart
+    if any of the dependencies' contents change OR if the target's content has
+    changed. SCons does not check the target's content, which can lead ot
+    inconsistenties.
+
+    To make this change transparent, any use of PyScript PyScons or
+    SetupPyScons will add this decider to the env. It is a hacky way to make
+    sure this decider is the default, so scripts don't need to be cahnged.
+    It is unclear how to do this another way.
+
+    Overiding using this decider as the default requires calling the Decider
+    function at the end of a SConstruct.
+    """
+    try:
+        if dep.get_csig() != prev_ni.csig:
+            return True # Rebuild if dependency has changed from sconsign
+    except AttributeError:
+        return True # rebuild if no csig for dependency
+
+    try:
+        if target.get_csig() != target.get_stored_info().ninfo.csig:
+            return True # rebuild if target has changed from sconsign file
+    except AttributeError:
+        return True # reuuild if no csig for target
+
+    return False # otherwise don't rebuild
+
 def mkdir_p(path):
     try:
         os.makedirs(path)
@@ -190,7 +220,7 @@ class PyScanner(MF):
             for f in D:
                 f = os.path.join(d,f)
                 if os.path.exists(f): imports.append(f)
-        
+
         imports = list(os.path.abspath(m) for m in imports if self.path_filter(m))
         return imports
 
@@ -201,13 +231,30 @@ def ToList(input):
 
 _USED_DIRS = set()
 
+_scons_template = """from pyscons import *
+targets, sources, basename = PySconsSetup()
+
+env = DefaultEnvironment()
+
+command_sig = %s
+
+stargets = [t + ".shadow" for t in targets]
+command =  [env.subst(c, target=stargets, source=sources) for c in %s ]
+command += ["mv %%s %%s" %% (s, t) for s,t in zip(stargets, targets)]
+
+gen = lambda source, target, env, for_signature: command_sig if for_signature else command
+PyScript(targets, sources, CommandGeneratorAction(gen,{}), clean=False)
+"""
+
+
 def _PyScript(env, target, source, command = "python $SOURCES", capture=None,
         execute=None, tempdir=None, exit=False, clean=True, lightweight=True, *args, **kwargs): 
     """
     A new Builder added to a scons environment which has the same syntax as Command.
     """
+    env.Decider(decider_tgt_src_content)
     target = ToList(target)
-    source = ToList(source) 
+    source = ToList(source)
     subst = list(source)
     command = ToList(command)
 
@@ -241,20 +288,7 @@ def _PyScript(env, target, source, command = "python $SOURCES", capture=None,
         if os.path.exists(basename): env.Execute(Delete(basename))
     elif execute:
         if not os.path.exists(basename): mkdir_p(basename)
-        scons = """from pyscons import *
-targets, sources, basename = PySconsSetup()
-
-env = DefaultEnvironment()
-
-command_sig = %s
-
-stargets = [t + ".shadow" for t in targets]
-command =  [env.subst(c, target=stargets, source=sources) for c in %s ]
-command += ["mv %%s %%s" %% (s, t) for s,t in zip(stargets, targets)]
-
-gen = lambda source, target, env, for_signature: command_sig if for_signature else command
-PyScript(targets, sources, CommandGeneratorAction(gen,{}), clean=False)
-""" % (repr(command_sig), repr(command))
+        scons = _scons_template % (repr(command_sig), repr(command))
         scons = re.sub("\$", "\$", scons)
 
         arg = " ".join(["in%g=%s" % (n+1, str(s)) for n,s in enumerate(source)])
@@ -285,6 +319,7 @@ PyScript(targets, sources, CommandGeneratorAction(gen,{}), clean=False)
 def _PyScons(env, target, source, scons, execute=None, options="-Q",
         sig_options="", clean = True, tempdir = None, exit=False):
     global _USED_DIRS
+    env.Decider(decider_tgt_src_content)
     target = ToList(target)
     source = ToList(source)
 
@@ -353,7 +388,7 @@ class PYTOOL:
 
 def PySconsSetup(env=None, args=ARGUMENTS, tools=['default', PYTOOL()]):
     if env == None: env = DefaultEnvironment(ENV=os.environ, tools=tools, IMPLICIT_COMMAND_DEPENDENCIES=0)
-
+    env.Decider(decider_tgt_src_content)
     sources = []
     targets = []
 
